#if !defined(__SHMEMM_H)
    #define __SHMEM_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Program name    : shmem.h                                                  */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   constants for the shared memory functions                */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Jun 2001                     */
/*                                                                            */
/******************************************************************************/

#include <rc.h>
#if 0
#if defined(WIN32)
	#include <windows.h>
	#include <winbase.h>
#endif
#endif

#include <trace_publ.h>

#if defined(__THREADED)
    #include <threads.h>
#endif

/* constants */
#if !defined(IN)
    #define IN
#endif
#if !defined(OUT)
    #define OUT
#endif
#if !defined(INOUT)
    #define INOUT
#endif
#if !defined(TRUE)
    #define TRUE 1
#endif
#if !defined(FALSE)
    #define FALSE 0
#endif
#if !defined(bool_t)
    #define bool_t int
#endif

#if !defined(SLEEP_THREAD_MILLISECONDS)
	#if defined(WIN32)
		#define SLEEP_THREAD_MILLISECONDS(__interval) Sleep(__interval)
	#else
		#define SLEEP_THREAD_MILLISECONDS(__interval) usleep(__interval*1000)
	#endif
#endif

typedef unsigned long shmemHandle_t;

typedef struct shmemCB {
	processLockStorage_t globalLock;  // used to hold UNIX locks
	shmemHandle_t pidList;
	bool_t initialisationComplete;
} shmemCB_t;

#define NUM_SLOTS_INCREMENT 10
typedef struct pidList {
	int numSlots;
	int activePids[1];
} pidList_t;

/* return sequence numbers */
#define RSN_001     1
#define RSN_002     2
#define RSN_003     3
#define RSN_004     4
#define RSN_005     5
#define RSN_006     6
#define RSN_007     7
#define RSN_008     8
#define RSN_009     9
#define RSN_010     10
#define RSN_011     11
#define RSN_012     12
#define RSN_013     13
#define RSN_014     14
#define RSN_015     15
#define RSN_016     16
#define RSN_017     17
#define RSN_018     18
#define RSN_019     19
#define RSN_020     20
#define RSN_021     21
#define RSN_022     22
#define RSN_023     23
#define RSN_024     24
#define RSN_025     25
#define RSN_026     26
#define RSN_027     27
#define RSN_028     28
#define RSN_029     29
#define RSN_030     30
#define RSN_031     31
#define RSN_032     32
#define RSN_033     33
#define RSN_034     34
#define RSN_035     35
#define RSN_036     36
#define RSN_037     37
#define RSN_038     38
#define RSN_039     39
#define RSN_040     40
#define RSN_041     41
#define RSN_042     42
#define RSN_043     43
#define RSN_044     44
#define RSN_045     45
#define RSN_046     46
#define RSN_047     47
#define RSN_048     48
#define RSN_049     49
#define RSN_050     50

#if defined(BUILD_AGGREGATOR_PLUGIN)
	#define SHARED_MEMORY_LOCK_NAME "Aggregator_Shmem_Lock"
#elif defined(BUILD_POSTIT_PLUGIN)
	#define SHARED_MEMORY_LOCK_NAME "Postit_Shmem_Lock"
#elif defined(BUILD_TIMER_PLUGIN)
	#define SHARED_MEMORY_LOCK_NAME "Timer_Shmem_Lock"
#elif defined(BUILD_UUIDGEN_PLUGIN)
	#define SHARED_MEMORY_LOCK_NAME "Uuidgen_Shmem_Lock"
#elif defined(BUILD_GETSQLCODE_PLUGIN)
	#define SHARED_MEMORY_LOCK_NAME "GetSQLCode_Shmem_Lock"
#else
	#define SHARED_MEMORY_LOCK_NAME "Default_Shmem_Lock"
#endif


#if defined(GLOBAL_SHMEM_DATA)
	void *sharedMemorySegment_Broker = NULL,
	     *sharedMemorySegment_Platform = NULL,
		 *heapP_Broker = NULL,
		 *heapP_Platform = NULL;
	shmemCB_t *shmemCBP_Broker = NULL,
		      *shmemCBP_Platform = NULL;
	processLock_t shmemLock_Broker = 0,  /* inter-process locks for shared memory segments */
	              shmemLock_Platform = 0;
#if defined(WIN32)
    HANDLE mapObjectHandle = NULL;  /* handle to shared memory file mapping */
#elif defined(_AIX) || defined(_SOLARIS) || defined(_HPUX)
	int sharedMemoryId_Broker = 0,
		sharedMemoryId_Platform = 0;
#endif
#else
	extern void *sharedMemorySegment_Broker,
				*sharedMemorySegment_Platform,
		        *heapP_Broker,
				*heapP_Platform;
	extern shmemCB_t *shmemCBP_Broker,
		             *shmemCBP_Platform;
	extern processLock_t shmemLock_Broker, /* inter-process locks for shared memory segments */
	                     shmemLock_Platform; 
#if defined(WIN32)
    extern HANDLE mapObjectHandle;  /* handle to shared memory file mapping */
#elif defined(_AIX) || defined(_SOLARIS)
	extern int sharedMemoryId_Broker,
	           sharedMemoryId_Platform;
#endif
#endif

#define SHM_NULL_HANDLE ((unsigned long)0xffffffff)

/* macros for converting handles to addresses and vice versa */
#define SHM_HANDLE2ADDRESS(__handle, __type, __sharedMemorySegment) \
    (__handle==SHM_NULL_HANDLE ? ((__type)NULL) : ((__type)(__handle + (unsigned long)(__sharedMemorySegment))))

#define SHM_ADDRESS2HANDLE(__address, __sharedMemorySegment) \
    ((void *)__address==NULL ? 0xffffffff : ((unsigned long)__address - (unsigned long)(__sharedMemorySegment)))


/* Function Prototypes */
void *shmMalloc(IN int memSize,
				IN bool_t lockAlreadyTaken,
				IN bool_t scopeIsBroker,
			    OUT int *rcP);

void *shmRealloc(IN void *oldAddressP,
				 IN int memSize,
				 IN bool_t lockAlreadyTaken,
				 IN bool_t scopeIsBroker,
			     OUT int *rcP);

int shmFree(IN void *oldAddressP,
			IN bool_t lockAlreadyTaken,
 		    IN bool_t scopeIsBroker);

int shmFreePidList(IN bool_t lockAlreadyTaken,
			       IN bool_t scopeIsBroker);

int shmResetWIP(IN int pid,
		        IN bool_t lockAlreadyTaken,
			    IN bool_t scopeIsBroker);

int shmAttach(IN char *segmentName,
		      IN int segmentSize,
			  IN void *CBInitialisationData,
			  IN int CBLength,
			  IN int currentPID,
			  IN char *segmentOwnerKey,
			  IN bool_t scopeIsBroker);

int shmDetach(IN int currentPID,
			  IN bool_t scopeIsBroker);

int shmLock(IN bool_t scopeIsBroker);

int shmUnlock(IN bool_t scopeIsBroker);

void *shmGetCBAddress(IN bool_t scopeIsBroker);

int shmGetPercentFree(IN bool_t lockAlreadyTaken,
			          IN bool_t scopeIsBroker);

void *shmQCAlloc(IN char QCName,
			     OUT int *rcP);

int shmQCFree(IN void *oldAddressP);

void shmDumpHeap(IN char *headerTextString,
				 IN bool_t scopeIsBroker);

int shmRemovePidFromList(IN int pid,
		  			     IN bool_t lockAlreadyTaken,
						 IN bool_t scopeIsBroker,
					     OUT bool_t *lastPIDP);

bool_t shmIsOnlyPidInList(IN int pid,
					      IN bool_t lockAlreadyTaken,
				          IN bool_t scopeIsBroker);

int shmRemoveAllPidsFromList(IN bool_t lockAlreadyTaken,
				             IN bool_t scopeIsBroker);


#endif

